@@ -20,7 +20,6 @@ pub struct PipeContext {
 }
 
 unsafe impl Send for PipeContext {}
-unsafe impl Sync for PipeContext {}
 
 #[derive(Clone, Copy)]
 #[repr(u32)]
@@ -40,7 +39,7 @@ impl PipeContext {
     pub(super) fn new(context: *mut pipe_context, screen: &Arc<PipeScreen>) -> Option<Self> {
         let s = Self {
             pipe: NonNull::new(context)?,
-            screen: screen.clone(),
+            screen: Arc::clone(screen),
         };
 
         if !has_required_cbs(unsafe { s.pipe.as_ref() }) {
@@ -51,6 +50,10 @@ impl PipeContext {
         Some(s)
     }
 
+    pub(crate) fn pipe(&self) -> NonNull<pipe_context> {
+        self.pipe
+    }
+
     pub fn buffer_subdata(
         &self,
         res: &PipeResource,
@@ -157,7 +160,7 @@ impl PipeContext {
         }
     }
 
-    pub fn resource_copy_region(
+    fn resource_copy_region(
         &self,
         src: &PipeResource,
         dst: &PipeResource,
@@ -179,6 +182,41 @@ impl PipeContext {
         }
     }
 
+    pub fn resource_copy_buffer(
+        &self,
+        src: &PipeResource,
+        src_offset: i32,
+        dst: &PipeResource,
+        dst_offset: u32,
+        width: i32,
+    ) {
+        debug_assert!(src.is_buffer());
+        debug_assert!(dst.is_buffer());
+
+        let bx = pipe_box {
+            x: src_offset,
+            width: width,
+            height: 1,
+            depth: 1,
+            ..Default::default()
+        };
+
+        self.resource_copy_region(src, dst, &[dst_offset, 0, 0], &bx)
+    }
+
+    pub fn resource_copy_texture(
+        &self,
+        src: &PipeResource,
+        dst: &PipeResource,
+        dst_offset: &[u32; 3],
+        bx: &pipe_box,
+    ) {
+        debug_assert!(!src.is_buffer());
+        debug_assert!(!dst.is_buffer());
+
+        self.resource_copy_region(src, dst, dst_offset, bx)
+    }
+
     fn resource_map(
         &self,
         res: &PipeResource,
@@ -205,12 +243,12 @@ impl PipeContext {
         }
     }
 
-    pub fn buffer_map(
+    pub fn buffer_map_flags(
         &self,
         res: &PipeResource,
         offset: i32,
         size: i32,
-        rw: RWFlags,
+        flags: pipe_map_flags,
     ) -> Option<PipeTransfer> {
         let b = pipe_box {
             x: offset,
@@ -220,20 +258,39 @@ impl PipeContext {
             ..Default::default()
         };
 
-        self.resource_map(res, &b, rw.into(), true)
+        self.resource_map(res, &b, flags, true)
+    }
+
+    pub fn buffer_map(
+        &self,
+        res: &PipeResource,
+        offset: i32,
+        size: i32,
+        rw: RWFlags,
+    ) -> Option<PipeTransfer> {
+        self.buffer_map_flags(res, offset, size, rw.into())
     }
 
     pub(super) fn buffer_unmap(&self, tx: *mut pipe_transfer) {
         unsafe { self.pipe.as_ref().buffer_unmap.unwrap()(self.pipe.as_ptr(), tx) };
     }
 
+    pub fn texture_map_flags(
+        &self,
+        res: &PipeResource,
+        bx: &pipe_box,
+        flags: pipe_map_flags,
+    ) -> Option<PipeTransfer> {
+        self.resource_map(res, bx, flags, false)
+    }
+
     pub fn texture_map(
         &self,
         res: &PipeResource,
         bx: &pipe_box,
         rw: RWFlags,
     ) -> Option<PipeTransfer> {
-        self.resource_map(res, bx, rw.into(), false)
+        self.texture_map_flags(res, bx, rw.into())
     }
 
     pub(super) fn texture_unmap(&self, tx: *mut pipe_transfer) {
@@ -250,7 +307,10 @@ impl PipeContext {
         unsafe { self.pipe.as_ref().create_compute_state.unwrap()(self.pipe.as_ptr(), &state) }
     }
 
-    pub fn bind_compute_state(&self, state: *mut c_void) {
+    /// # Safety
+    ///
+    /// The state pointer needs to point to valid memory until a new one is set.
+    pub unsafe fn bind_compute_state(&self, state: *mut c_void) {
         unsafe { self.pipe.as_ref().bind_compute_state.unwrap()(self.pipe.as_ptr(), state) }
     }
 
@@ -397,20 +457,11 @@ impl PipeContext {
         variable_local_mem: u32,
     ) {
         let info = pipe_grid_info {
-            pc: 0,
-            input: ptr::null(),
             variable_shared_mem: variable_local_mem,
             work_dim: work_dim,
             block: block,
-            last_block: [0; 3],
             grid: grid,
-            grid_base: [0; 3],
-            indirect: ptr::null_mut(),
-            indirect_offset: 0,
-            indirect_stride: 0,
-            draw_count: 0,
-            indirect_draw_count_offset: 0,
-            indirect_draw_count: ptr::null_mut(),
+            ..Default::default()
         };
         unsafe { self.pipe.as_ref().launch_grid.unwrap()(self.pipe.as_ptr(), &info) }
     }
@@ -428,25 +479,6 @@ impl PipeContext {
         }
     }
 
-    pub fn create_sampler_view(
-        &self,
-        res: &PipeResource,
-        format: pipe_format,
-        app_img_info: Option<&AppImgInfo>,
-    ) -> *mut pipe_sampler_view {
-        let template = res.pipe_sampler_view_template(format, app_img_info);
-
-        unsafe {
-            let s_view = self.pipe.as_ref().create_sampler_view.unwrap()(
-                self.pipe.as_ptr(),
-                res.pipe(),
-                &template,
-            );
-
-            s_view
-        }
-    }
-
     pub fn clear_global_binding(&self, count: u32) {
         unsafe {
             self.pipe.as_ref().set_global_binding.unwrap()(
@@ -459,7 +491,7 @@ impl PipeContext {
         }
     }
 
-    pub fn set_sampler_views(&self, views: &mut [*mut pipe_sampler_view]) {
+    pub fn set_sampler_views(&self, mut views: Vec<PipeSamplerView>) {
         unsafe {
             self.pipe.as_ref().set_sampler_views.unwrap()(
                 self.pipe.as_ptr(),
@@ -467,9 +499,8 @@ impl PipeContext {
                 0,
                 views.len() as u32,
                 0,
-                false,
-                views.as_mut_ptr(),
-            )
+                PipeSamplerView::as_pipe(views.as_mut_slice()),
+            );
         }
     }
 
@@ -482,16 +513,11 @@ impl PipeContext {
                 0,
                 count,
                 0,
-                false,
                 samplers.as_mut_ptr(),
             )
         }
     }
 
-    pub fn sampler_view_destroy(&self, view: *mut pipe_sampler_view) {
-        unsafe { self.pipe.as_ref().sampler_view_destroy.unwrap()(self.pipe.as_ptr(), view) }
-    }
-
     pub fn set_shader_images(&self, images: &[PipeImageView]) {
         let images = PipeImageView::slice_to_pipe(images);
         unsafe {
@@ -595,6 +621,16 @@ impl PipeContext {
             }
         }
     }
+
+    pub fn device_reset_status(&self) -> pipe_reset_status {
+        unsafe {
+            if let Some(get_device_reset_status) = self.pipe.as_ref().get_device_reset_status {
+                get_device_reset_status(self.pipe.as_ptr())
+            } else {
+                pipe_reset_status::PIPE_NO_RESET
+            }
+        }
+    }
 }
 
 impl Drop for PipeContext {
@@ -626,10 +662,12 @@ fn has_required_cbs(context: &pipe_context) -> bool {
         & has_required_feature!(context, launch_grid)
         & has_required_feature!(context, memory_barrier)
         & has_required_feature!(context, resource_copy_region)
+        // implicitly used through pipe_sampler_view_reference
         & has_required_feature!(context, sampler_view_destroy)
         & has_required_feature!(context, set_constant_buffer)
         & has_required_feature!(context, set_global_binding)
         & has_required_feature!(context, set_sampler_views)
+        & has_required_feature!(context, sampler_view_release)
         & has_required_feature!(context, set_shader_images)
         & has_required_feature!(context, texture_map)
         & has_required_feature!(context, texture_subdata)
